// SPDX-License-Identifier: MPL-2.0
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  applyPlugins: () => applyPlugins,
  check: () => check,
  checkCommit: () => checkCommit,
  checkHeader: () => checkHeader,
  cleaner: () => cleaner,
  errorMsg: () => errorMsg,
  isBreakingChange: () => isBreakingChange,
  isValidString: () => isValidString,
  mappers: () => mappers,
  normalizeCommit: () => normalizeCommit,
  parse: () => parse,
  parseCommit: () => parseCommit,
  parseHeader: () => parseHeader,
  plugins: () => plugins,
  stringToHeader: () => stringToHeader,
  stringify: () => stringify,
  stringifyCommit: () => stringifyCommit,
  stringifyHeader: () => stringifyHeader,
  toArray: () => toArray,
  validate: () => validate,
  validateCommit: () => validateCommit,
  validateHeader: () => validateHeader
});
module.exports = __toCommonJS(src_exports);
var import_mixin_deep = __toESM(require("mixin-deep"), 1);

// src/utils.ts
var errorMsg = `parse-commit-message: expect \`commit\` to follow:
<type>[optional scope]: <description>

[optional body]

[optional footer]`;
function isValidString(x) {
  return Boolean(x && typeof x === "string" && x.length > 0);
}
function stringToHeader(val, options) {
  if (typeof val !== "string") {
    throw new TypeError("stringToHeader: expects `val` to be a string");
  }
  const opts = { caseSensitive: false, ...options };
  const defaultRegex = opts.caseSensitive ? /^(\w+)(?:\((.+)\))?(!)?: (.+)/ : /^(\w+)(?:\((.+)\))?(!)?: (.+)/i;
  let regex = null;
  if (opts.headerRegex && typeof opts.headerRegex === "string") {
    regex = opts.caseSensitive ? new RegExp(opts.headerRegex) : new RegExp(opts.headerRegex, "i");
  }
  if (opts.headerRegex && opts.headerRegex instanceof RegExp) {
    regex = opts.headerRegex;
  }
  if (!regex) {
    regex = defaultRegex;
  }
  if (!regex.test(val)) {
    throw new TypeError(errorMsg);
  }
  const matches = regex?.exec(val)?.slice(1);
  if (!matches) {
    throw new Error(errorMsg);
  }
  const [type, scope = null, bang = "", subject = ""] = matches;
  return {
    type: type + bang,
    scope,
    subject
  };
}
function normalizeCommit(commit, options) {
  const { header } = commit;
  if (header && typeof header === "object" && "value" in header && typeof header.value === "string") {
    return { ...commit, header: stringToHeader(header.value, options) };
  }
  return commit;
}
function toArray(val) {
  if (!val) return [];
  return [val];
}
function isBreakingChange(commit) {
  const re = /^BREAKING\s+CHANGES?:\s+/;
  return (
    // give the bang precedence since it's the official way
    commit.header.type.endsWith("!") || /break|breaking|major/i.test(commit.header.type) || re.test(commit.header.subject) || re.test(commit.body || "") || re.test(commit.footer || "")
  );
}
function cleaner(x) {
  return {
    ...x,
    header: {
      ...x.header,
      type: x.header.type.endsWith("!") ? x.header.type.slice(0, -1) : x.header.type
    }
  };
}

// src/plugins/increment.ts
function incrementPlugin(commit, options) {
  const opts = { normalize: true, ...options };
  const cmt = opts.normalize ? normalizeCommit(commit, opts) : commit;
  const isBreaking = isBreakingChange(cmt);
  let commitIncrement = false;
  if (/fix|bugfix|patch/i.test(cmt.header.type)) {
    commitIncrement = "patch";
  }
  if (/feat|feature|minor/i.test(cmt.header.type)) {
    commitIncrement = "minor";
  }
  if (isBreaking) {
    commitIncrement = "major";
  }
  if (!commitIncrement) {
    return {};
  }
  return { increment: commitIncrement };
}

// src/plugins/is-breaking-change.ts
function isBreakingChangePlugin(commit, options) {
  const opts = { normalize: true, ...options };
  const cmt = opts.normalize ? normalizeCommit(commit, opts) : commit;
  const isBreaking = isBreakingChange(cmt);
  return {
    isBreakingChange: isBreaking,
    isBreaking
    // ! deprecated
  };
}

// src/plugins/mentions.ts
var import_collect_mentions = __toESM(require("collect-mentions"), 1);
function mentionsPlugin(commit, options) {
  const opts = { normalize: true, mentionsWithDot: false, ...options };
  const cmt = opts.normalize ? normalizeCommit(commit, opts) : commit;
  const commitMentions = [
    (0, import_collect_mentions.default)(
      cmt.header?.subject || cmt.header?.value || "",
      opts.mentionsWithDot
    )
  ].flat().concat((0, import_collect_mentions.default)(cmt.body || "", opts.mentionsWithDot)).concat((0, import_collect_mentions.default)(cmt.footer || "", opts.mentionsWithDot));
  return { mentions: commitMentions };
}

// src/header.ts
function parseHeader(header, options) {
  if (!isValidString(header)) {
    throw new TypeError("expect `header` to be non empty string");
  }
  return stringToHeader(header.trim(), options);
}
function stringifyHeader(header, options) {
  const res = validateHeader(header, options);
  if (res.error) {
    throw res.error;
  }
  if (res.value && typeof res.value === "object" && "value" in res.value) {
    return res.value.value;
  }
  const { type, scope, subject } = res.value;
  return `${type}${scope ? `(${scope})` : ""}: ${subject}`.trim();
}
function validateHeader(header, options) {
  const result = {};
  try {
    result.value = checkHeader(header, options);
  } catch (err) {
    return { error: err };
  }
  return result;
}
function checkHeader(header, options) {
  if (header && typeof header === "object" && "value" in header) {
    const { value } = header;
    return stringToHeader(value, options);
  }
  if (!isValidString(header?.type)) {
    throw new TypeError("header.type should be non empty string");
  }
  if (!isValidString(header?.subject)) {
    throw new TypeError("header.subject should be non empty string");
  }
  const isValidScope = header && "scope" in header && header.scope !== null ? isValidString(header.scope) : true;
  if (!isValidScope) {
    throw new TypeError("commit.header.scope should be non empty string when given");
  }
  return { scope: null, ...header };
}

// src/commit.ts
function parseCommit(commit, options) {
  if (!isValidString(commit)) {
    throw new TypeError(`expect \`commit\` to be non empty string`);
  }
  const header = parseHeader(commit, options);
  const [body = null, footer = null] = commit.split("\n\n").slice(1);
  return { header, body, footer };
}
function stringifyCommit(commit, options) {
  const result = validateCommit(commit, options);
  if (result.error || !result.value) {
    const err = result.error || new TypeError("expect `commit` to be a valid object");
    throw err;
  }
  const header = stringifyHeader(result.value.header, options);
  const EOL = "\n";
  result.value.body = result.value.body ? EOL + EOL + result.value.body : "";
  result.value.footer = result.value.footer ? EOL + EOL + result.value.footer : "";
  return `${header}${result.value.body}${result.value.footer}`;
}
function validateCommit(commit, options) {
  const result = {};
  try {
    result.value = checkCommit(commit, options);
  } catch (err) {
    return { error: err };
  }
  return result;
}
function checkCommit(commit, options) {
  const { error, value: headerObj } = validateHeader(commit.header, options);
  if (error) {
    throw error;
  }
  const isValidBody = "body" in commit && commit.body !== null ? typeof commit.body === "string" : true;
  if (!isValidBody) {
    throw new TypeError("commit.body should be string when given");
  }
  const isValid = "footer" in commit && commit.footer !== null ? typeof commit.footer === "string" : true;
  if (!isValid) {
    throw new TypeError("commit.footer should be string when given");
  }
  return { body: null, footer: null, ...commit, header: headerObj };
}

// src/main.ts
function parse(commits, options) {
  const result = toArray(commits).filter(Boolean).flat().reduce((acc, val) => {
    if (typeof val === "string") {
      return acc.concat(parseCommit(val, options));
    }
    if (typeof val === "object" && !Array.isArray(val)) {
      return acc.concat(val);
    }
    return acc.concat(parse(val, options));
  }, []);
  return result;
}
function stringify(commits, options) {
  const result = toArray(commits).filter(Boolean).flat().reduce((acc, val) => {
    const xxx = typeof val === "string" ? { header: { value: val } } : val;
    const foo = check(xxx, options);
    const bar = toArray(foo).map((x) => stringifyCommit(x, options));
    return acc.concat(bar);
  }, []);
  return result;
}
function validate(commits, options) {
  const result = {};
  try {
    result.value = check(commits, options);
  } catch (err) {
    return { error: err };
  }
  return result;
}
function check(commits, options) {
  const result = toArray(commits).filter(Boolean).flat().reduce((acc, commit) => {
    if (typeof commit === "string") {
      commit = parseCommit(commit, options);
    }
    return acc.concat(checkCommit(commit, options));
  }, []);
  if (result.length === 0) {
    throw new Error(errorMsg);
  }
  return result;
}

// src/index.ts
function applyPlugins(plugins2, commits, options) {
  const opts = { caseSensitive: false, normalize: true, ...options };
  const arr = [];
  const cmts = [];
  const plgs = arr.concat(plugins2).filter(Boolean);
  return cmts.concat(commits).filter(Boolean).reduce((result, commit) => {
    let commitObject = {};
    if (typeof commit === "string") {
      commitObject = { header: { value: commit } };
    } else if (typeof commit === "object" && !Array.isArray(commit)) {
      commitObject = commit;
    }
    const cmt = plgs.reduce((acc, fn) => {
      const res = fn(acc, opts);
      return (0, import_mixin_deep.default)(acc, res);
    }, commitObject);
    return result.concat(cmt);
  }, []);
}
var plugins = [mentionsPlugin, incrementPlugin, isBreakingChangePlugin];
var mappers = {
  mentions: mentionsPlugin,
  increment: incrementPlugin,
  isBreaking: isBreakingChangePlugin,
  isBreakingChange: isBreakingChangePlugin
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  applyPlugins,
  check,
  checkCommit,
  checkHeader,
  cleaner,
  errorMsg,
  isBreakingChange,
  isValidString,
  mappers,
  normalizeCommit,
  parse,
  parseCommit,
  parseHeader,
  plugins,
  stringToHeader,
  stringify,
  stringifyCommit,
  stringifyHeader,
  toArray,
  validate,
  validateCommit,
  validateHeader
});
